@@ -117,7 +117,7 @@ local function set_ctx_and_check_token()
     local ok, err = check_token(api_ctx)
     if not ok then
         core.log.warn("failed to check token: ", err)
-        core.response.exit(401, { error_msg = "failed to check token" })
+        core.response.exit(401, { error_msg = "failed to check token", description = err })
     end
 end
 
@@ -281,7 +281,7 @@ end
 local function post_reload_plugins()
     set_ctx_and_check_token()
 
-    local success, err = events.post(reload_event, get_method(), ngx_time())
+    local success, err = events:post(reload_event, get_method(), ngx_time())
     if not success then
         core.response.exit(503, err)
     end
@@ -453,16 +453,17 @@ function _M.init_worker()
     end
 
     router = route.new(uri_route)
-    events = require("resty.worker.events")
 
-    events.register(reload_plugins, reload_event, "PUT")
+    -- register reload plugin handler
+    events = require("apisix.events")
+    events:register(reload_plugins, reload_event, "PUT")
 
     if ngx_worker_id() == 0 then
         -- check if admin_key is required
         if local_conf.deployment.admin.admin_key_required == false then
             core.log.warn("Admin key is bypassed! ",
                 "If you are deploying APISIX in a production environment, ",
-                "please disable `admin_key_required` and set a secure admin key!")
+                "please enable `admin_key_required` and set a secure admin key!")
         end
 
         local ok, err = ngx_timer_at(0, function(premature)
