@@ -15,15 +15,15 @@
 -- limitations under the License.
 --
 
-local yaml = require("tinyyaml")
+local yaml = require("lyaml")
 local profile = require("apisix.core.profile")
 local util = require("apisix.cli.util")
+local default_conf = require("apisix.cli.config")
 local dkjson = require("dkjson")
 
 local pairs = pairs
 local type = type
 local tonumber = tonumber
-local getmetatable = getmetatable
 local getenv = os.getenv
 local str_gmatch = string.gmatch
 local str_find = string.find
@@ -157,14 +157,6 @@ local function replace_by_reserved_env_vars(conf)
 end
 
 
-local function tinyyaml_type(t)
-    local mt = getmetatable(t)
-    if mt then
-        return mt.__type
-    end
-end
-
-
 local function path_is_multi_type(path, type_val)
     if str_sub(path, 1, 14) == "nginx_config->" and
             (type_val == "number" or type_val == "string") then
@@ -175,7 +167,7 @@ local function path_is_multi_type(path, type_val)
         return true
     end
 
-    if path == "apisix->ssl->key_encrypt_salt" then
+    if path == "apisix->data_encryption->keyring" then
         return true
     end
 
@@ -188,7 +180,7 @@ local function merge_conf(base, new_tab, ppath)
 
     for key, val in pairs(new_tab) do
         if type(val) == "table" then
-            if tinyyaml_type(val) == "null" then
+            if val == yaml.null then
                 base[key] = nil
 
             elseif tab_is_array(val) then
@@ -237,18 +229,7 @@ function _M.read_yaml_conf(apisix_home)
         profile.apisix_home = apisix_home .. "/"
     end
 
-    local local_conf_path = profile:yaml_path("config-default")
-    local default_conf_yaml, err = util.read_file(local_conf_path)
-    if not default_conf_yaml then
-        return nil, err
-    end
-
-    local default_conf = yaml.parse(default_conf_yaml)
-    if not default_conf then
-        return nil, "invalid config-default.yaml file"
-    end
-
-    local_conf_path = profile:customized_yaml_path()
+    local local_conf_path = profile:customized_yaml_path()
     if not local_conf_path then
         local_conf_path = profile:yaml_path("config")
     end
@@ -266,7 +247,7 @@ function _M.read_yaml_conf(apisix_home)
     end
 
     if not is_empty_file then
-        local user_conf = yaml.parse(user_conf_yaml)
+        local user_conf = yaml.load(user_conf_yaml)
         if not user_conf then
             return nil, "invalid config.yaml file"
         end
@@ -306,7 +287,7 @@ function _M.read_yaml_conf(apisix_home)
         local apisix_conf_path = profile:yaml_path("apisix")
         local apisix_conf_yaml, _ = util.read_file(apisix_conf_path)
         if apisix_conf_yaml then
-            local apisix_conf = yaml.parse(apisix_conf_yaml)
+            local apisix_conf = yaml.load(apisix_conf_yaml)
             if apisix_conf then
                 local ok, err = resolve_conf_var(apisix_conf)
                 if not ok then
