@@ -25,11 +25,14 @@ local collectgarbage = collectgarbage
 local ipairs = ipairs
 local pcall = pcall
 local str_format = string.format
+local ngx = ngx
 local ngx_var = ngx.var
+local events = require("apisix.events")
 
 
 local _M = {}
 
+_M.RELOAD_EVENT = 'control-api-plugin-reload'
 
 function _M.schema()
     local http_plugins, stream_plugins = plugin.get_all({
@@ -130,7 +133,7 @@ local HTML_TEMPLATE = [[
 {% for _, stat in ipairs(stats) do %}
 {% for _, node in ipairs(stat.nodes) do %}
 {% i = i + 1 %}
-  {% if node.status == "healthy" then %}
+  {% if node.status == "healthy" or node.status == "mostly_healthy" then %}
   <tr>
   {% else %}
   <tr bgcolor="#FF0000">
@@ -400,6 +403,14 @@ function _M.dump_plugin_metadata()
     return 200, metadata.value
 end
 
+function _M.post_reload_plugins()
+    local success, err = events:post(_M.RELOAD_EVENT, ngx.req.get_method(), ngx.time())
+    if not success then
+        core.response.exit(503, err)
+    end
+
+    core.response.exit(200, "done")
+end
 
 return {
     -- /v1/schema
@@ -474,5 +485,12 @@ return {
         uris = {"/plugin_metadata/*"},
         handler = _M.dump_plugin_metadata,
     },
+    -- /v1/plugins/reload
+    {
+        methods = {"PUT"},
+        uris = {"/plugins/reload"},
+        handler = _M.post_reload_plugins,
+    },
     get_health_checkers = _get_health_checkers,
+    reload_event = _M.RELOAD_EVENT,
 }
