@@ -154,16 +154,6 @@ local function kvs_to_node(kvs)
 end
 _M.kvs_to_node = kvs_to_node
 
-local function kvs_to_nodes(res)
-    res.body.node.dir = true
-    res.body.node.nodes = setmetatable({}, array_mt)
-    for i=2, #res.body.kvs do
-        res.body.node.nodes[i-1] = kvs_to_node(res.body.kvs[i])
-    end
-    return res
-end
-
-
 local function not_found(res)
     res.body.message = "Key not found"
     res.reason = "Not found"
@@ -211,13 +201,22 @@ function _M.get_format(res, real_key, is_dir, formatter)
     else
         -- In etcd v2, the direct key asked for is `node`, others which under this dir are `nodes`
         -- While in v3, this structure is flatten and all keys related the key asked for are `kvs`
-        res.body.node = kvs_to_node(res.body.kvs[1])
-        if not res.body.kvs[1].value then
-            -- remove last "/" when necessary
-            if string.byte(res.body.node.key, -1) == 47 then
-                res.body.node.key = string.sub(res.body.node.key, 1, #res.body.node.key-1)
+        res.body.node = {
+            key = real_key,
+            dir = true,
+            nodes = setmetatable({}, array_mt)
+        }
+        local kvs = res.body.kvs
+        if #kvs >= 1 and not kvs[1].value then
+            res.body.node.createdIndex = tonumber(kvs[1].create_revision)
+            res.body.node.modifiedIndex = tonumber(kvs[1].mod_revision)
+            for i=2, #kvs do
+                res.body.node.nodes[i-1] = kvs_to_node(kvs[i])
+            end
+        else
+            for i=1, #kvs do
+                res.body.node.nodes[i] = kvs_to_node(kvs[i])
             end
-            res = kvs_to_nodes(res)
         end
     end
 
@@ -316,7 +315,6 @@ function _M.get(key, is_dir)
     if not res then
         return nil, err
     end
-
     return _M.get_format(res, key, is_dir)
 end
 
