@@ -19,6 +19,7 @@ local core = require("apisix.core")
 local discovery = require("apisix.discovery.init").discovery
 local upstream_util = require("apisix.utils.upstream")
 local apisix_ssl = require("apisix.ssl")
+local events = require("apisix.events")
 local error = error
 local tostring = tostring
 local ipairs = ipairs
@@ -36,7 +37,7 @@ if ok then
     set_upstream_tls_client_param = apisix_ngx_upstream.set_cert_and_key
 else
     set_upstream_tls_client_param = function ()
-        return nil, "need to build APISIX-Base to support upstream mTLS"
+        return nil, "need to build APISIX-Runtime to support upstream mTLS"
     end
 end
 
@@ -47,7 +48,7 @@ if not is_http then
         set_stream_upstream_tls = apisix_ngx_stream_upstream.set_tls
     else
         set_stream_upstream_tls = function ()
-            return nil, "need to build APISIX-Base to support TLS over TCP upstream"
+            return nil, "need to build APISIX-Runtime to support TLS over TCP upstream"
         end
     end
 end
@@ -110,10 +111,18 @@ local function create_checker(upstream)
     end
     upstream.is_creating_checker = true
 
+    core.log.debug("events module used by the healthcheck: ", events.events_module,
+                    ", module name: ",events:get_healthcheck_events_modele())
+
     local checker, err = healthcheck.new({
         name = get_healthchecker_name(healthcheck_parent),
         shm_name = "upstream-healthcheck",
         checks = upstream.checks,
+        -- the events.init_worker will be executed in the init_worker phase,
+        -- events.healthcheck_events_module is set
+        -- while the healthcheck object is executed in the http access phase,
+        -- so it can be used here
+        events_module = events:get_healthcheck_events_modele(),
     })
 
     if not checker then
@@ -282,6 +291,8 @@ function _M.set_by_route(route, api_ctx)
             end
 
             up_conf.nodes = new_nodes
+            up_conf.original_nodes = up_conf.nodes
+
             local new_up_conf = core.table.clone(up_conf)
             core.log.info("discover new upstream from ", up_conf.service_name, ", type ",
                           up_conf.discovery_type, ": ",
@@ -440,6 +451,18 @@ local function check_upstream_conf(in_dp, conf)
             return false, "invalid configuration: " .. err
         end
 
+        if conf.nodes and not core.table.isarray(conf.nodes) then
+            local port
+            for addr,_ in pairs(conf.nodes) do
+                _, port = core.utils.parse_addr(addr)
+                if port then
+                    if port < 1 or port > 65535 then
+                        return false, "invalid port " .. tostring(port)
+                    end
+                end
+            end
+        end
+
         local ssl_id = conf.tls and conf.tls.client_cert_id
         if ssl_id then
             local key = "/ssls/" .. ssl_id
